/* Copyright © 2020 Purism SPC
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 */

/**
 * Cell renderer for the expander in the sidebar.
 */
public class SidebarExpanderRenderer : Gtk.CellRendererPixbuf {
    public signal void toggle(Gtk.TreePath path);
    public weak Gtk.Widget widget { get; set; }
    public SidebarExpanderRenderer(Gtk.Widget widget) {
        this.widget = widget;
        xalign = 1;
        mode = Gtk.CellRendererMode.ACTIVATABLE;
        widget.style_updated.connect(load_expander_arrow);
        load_expander_arrow();
    }

    public override bool activate (Gdk.Event event,
                                   Gtk.Widget widget,
                                   string path,
                                   Gdk.Rectangle background_area,
                                   Gdk.Rectangle cell_area,
                                   Gtk.CellRendererState flags) {
        toggle(new Gtk.TreePath.from_string (path));
        return true;
    }

    private void load_expander_arrow() {
        Gtk.IconLookupFlags flags = Gtk.IconLookupFlags.DIR_LTR;
        Gdk.PixbufRotation rotation = Gdk.PixbufRotation.COUNTERCLOCKWISE;
        if (widget.get_direction() == Gtk.TextDirection.RTL) {
            flags = Gtk.IconLookupFlags.DIR_RTL;
            rotation = Gdk.PixbufRotation.CLOCKWISE;
        }

        int size;
        Gtk.Requisition minimum_size, natural_size;
        get_preferred_size(widget, out minimum_size, out natural_size);

        if (minimum_size.height > natural_size.height) {
            size = minimum_size.height;
        } else {
            size = natural_size.height;
        }

        try {
            Gtk.IconInfo arrow_info = Gtk.IconTheme.get_default()
                .lookup_icon_for_scale("expander-arrow-symbolic",
                                       size,
                                       widget.get_scale_factor(),
                                       flags);
            var context = widget.get_style_context();
            var color = context.get_color(context.get_state());

            var arrow_icon = arrow_info.load_symbolic(color);
            pixbuf_expander_open = arrow_icon;
            pixbuf_expander_closed = arrow_icon.rotate_simple(rotation);
        } catch (Error e) {
            message("Couldn't load icon: %s", e.message);
        }
    }
}

